# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-26 19:37
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_slack_oauth', '0003_slackuser_accesstoken_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='SlackOAuthRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('access_token', models.CharField(blank=True, max_length=128, null=True)),
                ('extras', jsonfield.fields.JSONField(blank=True, null=True)),
                ('ip', models.GenericIPAddressField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now_add=True)),
                ('associated_user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL)
    ]

    operations = [
        migrations.CreateModel(
            name='SlackUser',
            fields=[
                ('slacker', models.OneToOneField(related_name='slack_user', primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('access_token', models.CharField(max_length=64, null=True)),
            ],
            options={
                'db_table': 'slack_user',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.urlresolvers import reverse_lazy

__all__ = (
    'settings',
)


default_settings = {
    'SLACK_CLIENT_ID': None,
    'SLACK_CLIENT_SECRET': None,

    'SLACK_AUTHORIZATION_URL': 'https://slack.com/oauth/authorize',
    'SLACK_OAUTH_ACCESS_URL': 'https://slack.com/api/oauth.access',
    'SLACK_SUCCESS_REDIRECT_URL': reverse_lazy('slack_success'),
    'SLACK_ERROR_REDIRECT_URL': '/',

    'SLACK_SCOPE': 'identify,read,post',

    'SLACK_PIPELINES': [
        'django_slack_oauth.pipelines.log_request',
        'django_slack_oauth.pipelines.slack_user'
    ]
}


class Settings(object):
    def __init__(self, app_settings, defaults):
        for k, v in defaults.items():
            setattr(self, k, getattr(app_settings, k, v))

settings = Settings(settings, default_settings)




# -*- coding: utf-8 -*-

from django.conf.urls import url

from .views import SlackAuthView, DefaultSuccessView


urlpatterns = [
    url('login/', SlackAuthView.as_view(), name='slack_auth'),
    url('success/', DefaultSuccessView.as_view(), name='slack_success')
]



# -*- coding: utf-8 -*-

from copy import deepcopy

from django.contrib import messages
from .models import SlackOAuthRequest, SlackUser


__all__ = (
    'log_request',
    'slack_user',

    'debug',
)


def log_request(request, api_data):
    SlackOAuthRequest.objects.create(
        access_token=api_data.pop('access_token'),
        extras=api_data
    )

    return request, api_data


def slack_user(request, api_data):
    """
    Pipeline for backward compatibility prior to 1.0.0 version.
    In case if you're willing maintain `slack_user` table.

    """
    if request.user.is_anonymous:
        return request, api_data

    data = deepcopy(api_data)

    slacker, _ = SlackUser.objects.get_or_create(slacker=request.user)
    slacker.access_token = data.pop('access_token')
    slacker.extras = data
    slacker.save()

    messages.add_message(request, messages.SUCCESS, 'Your account has been successfully updated with '
                                                    'Slack. You can share your messages within your slack '
                                                    'domain.')

    return request, api_data


def debug(request, api_data):
    print("request: {}, api_data: {}".format(request, api_data))
    return request, api_data



# -*- coding: utf-8 -*-

from jsonfield import JSONField

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


__all__ = (
    'SlackUser',
    'SlackOAuthRequest',
)


@python_2_unicode_compatible
class SlackUser(models.Model):
    slacker = models.OneToOneField(settings.AUTH_USER_MODEL, primary_key=True, related_name='slack_user')
    access_token = models.CharField(max_length=128, null=True)
    extras = JSONField(null=True)

    def is_slacked(self):
        return self.access_token

    def __str__(self):
        return self.slacker.__str__()

    class Meta:
        db_table = 'slack_user'


class SlackOAuthRequest(models.Model):
    associated_user = models.OneToOneField(settings.AUTH_USER_MODEL, null=True, blank=True)
    access_token = models.CharField(max_length=128, null=True, blank=True)
    extras = JSONField(null=True, blank=True)

    ip = models.GenericIPAddressField(null=True, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now_add=True)

    class Meta:
        verbose_name = "Slack OAuth Request"
        verbose_name_plural = "Slack OAuth Requests"



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-28 20:55
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_slack_oauth', '0004_auto_20161226_1937'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='slackoauthrequest',
            options={'verbose_name': 'Slack OAuth Request', 'verbose_name_plural': 'Slack OAuth Requests'},
        ),
    ]



# -*- coding: utf-8 -*-

import uuid
from importlib import import_module

import requests

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.core.cache import cache
from django.http.response import HttpResponseRedirect, HttpResponse
from django.views.generic import RedirectView, View

from . import settings

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

__all__ = (
    'SlackAuthView',
    'DefaultSuccessView'
)


class StateMismatch(Exception):
    pass


class DefaultSuccessView(View):
    def get(self, request):
        messages.success(request, "You've been successfully authenticated.")
        return HttpResponse("Slack OAuth login successful.")


class SlackAuthView(RedirectView):
    permanent = True

    text_error = 'Attempt to update has failed. Please try again.'

    @property
    def cache_key(self):
        return 'slack:' + str(self.request.user)

    def get(self, request, *args, **kwargs):
        code = request.GET.get('code')
        if not code:
            return self.auth_request()

        self.validate_state(request.GET.get('state'))

        access_content = self.oauth_access(code)
        if not access_content.status_code == 200:
            return self.error_message()

        api_data = access_content.json()
        if not api_data['ok']:
            return self.error_message(api_data['error'])

        pipelines = settings.SLACK_PIPELINES

        # pipelines is a list of the callables to be executed
        pipelines = [getattr(import_module('.'.join(p.split('.')[:-1])), p.split('.')[-1]) for p in pipelines]
        return self.execute_pipelines(request, api_data, pipelines)

    def execute_pipelines(self, request, api_data, pipelines):
        if len(pipelines) == 0:
            # Terminate at the successful redirect
            return self.response()
        else:
            # Call the next function in the queue
            request, api_data = pipelines.pop(0)(request, api_data)
            return self.execute_pipelines(request, api_data, pipelines)

    def auth_request(self):
        state = self.store_state()

        params = urlencode({
            'client_id': settings.SLACK_CLIENT_ID,
            'redirect_uri': self.request.build_absolute_uri(reverse('slack_auth')),
            'scope': settings.SLACK_SCOPE,
            'state': state,
        })

        return self.response(settings.SLACK_AUTHORIZATION_URL + '?' + params)

    def oauth_access(self, code):
        params = {
            'client_id': settings.SLACK_CLIENT_ID,
            'client_secret': settings.SLACK_CLIENT_SECRET,
            'code': code,
            'redirect_uri': self.request.build_absolute_uri(reverse('slack_auth')),
        }

        return requests.get(settings.SLACK_OAUTH_ACCESS_URL, params=params)

    def validate_state(self, state):
        state_before = cache.get(self.cache_key)
        cache.delete(self.cache_key)
        if state_before != state:
            raise StateMismatch('State mismatch upon authorization completion.'
                                ' Try new request.')
        return True

    def store_state(self):
        state = str(uuid.uuid4())[:6]
        cache.set(self.cache_key, state)
        return state

    def error_message(self, msg=text_error):
        messages.add_message(self.request, messages.ERROR, '%s' % msg)
        return self.response(redirect=settings.SLACK_ERROR_REDIRECT_URL)

    def response(self, redirect=settings.SLACK_SUCCESS_REDIRECT_URL):
        return HttpResponseRedirect(redirect)



# -*- coding: utf-8 -*-

import os
from setuptools import setup


version = '1.3.0'


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-slack-oauth',
    version=version,
    packages=[
        'django_slack_oauth',
        'django_slack_oauth.migrations',
    ],
    include_package_data=True,
    license='MIT License',
    description='Handles OAuth and stores slack token',
    long_description=README,
    author='Sergey Keller',
    author_email='izdieu@gmail.com',
    url='https://github.com/izdi/django-slack-oauth',
    download_url='https://github.com/izdi/django-slack-oauth/tarball/1.0.0',
    install_requires=[
        'Django>=1.8',
        'requests',
        'jsonfield',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_slack_oauth', '0002_slackuser_extras'),
    ]

    operations = [
        migrations.AlterField(
            model_name='slackuser',
            name='access_token',
            field=models.CharField(max_length=128, null=True),
        ),
    ]
